from lido_sdk.network import Network


MULTICALL_ADDRESSES = {
    Network.Mainnet: "0xeefBa1e63905eF1D7ACbA5a8513c70307C1cE441",
    Network.Kovan: "0x2cc8688C5f75E365aaEEb4ea8D6a480405A48D2A",
    Network.Rinkeby: "0x42Ad527de7d4e9d9d011aC45B31D8551f8Fe9821",
    Network.Görli: "0x77dCa2C955b15e9dE4dbBCf1246B4B85b651e50e",
    Network.xDai: "0xb5b692a88BDFc81ca69dcB1d924f59f0413A602a",
    Network.Ropsten: "0x53c43764255c17bd724f74c4ef150724ac50a3ed",
}
